﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace ELFViewerConsole.ELFModel
{
    internal class BinaryFile
    {
        /************************************************************
         * Variables
         ************************************************************/
        private string filename;

        private FileDataHandler fdata;

        private FileInfo fi;


        /************************************************************
         * Properties
         ************************************************************/
        /// <summary>
        /// Name of file including extension
        /// </summary>
        public string FileName
        {
            get { return fi.Name; }
        }

        /// <summary>
        /// Name including complete filepath and extension of file
        /// </summary>
        public string FilePath
        {
            get { return fi.FullName; }
        }

        /// <summary>
        /// Length of file in bytes.
        /// </summary>
        public long FileSize
        {
            get
            {
                if (fi.Length != fdata.Length) { throw new InvalidOperationException(); }
                return fi.Length;
            }
        }

        public IFileAccess FileData
        {
            get { return fdata; }
        }

        public FileInfo FileInfo
        {
            get { return fi; }
        }

        /************************************************************
         * Methods
         ************************************************************/

        public BinaryFile(string filename)
        {
            this.filename = filename;
        }

        public virtual void LoadFile()
        {
            FileStream fs;

            if (filename == null)
            {
                throw new ArgumentNullException();
            }

            if (!File.Exists(filename))
            {
                throw new ArgumentException(filename + " does not exist.");
            }

            // get file info object
            fi = new FileInfo(filename);

            // create file stream 
            fs = new FileStream(filename, FileMode.Open, FileAccess.Read, FileShare.Read);

            // get total byte length of the file
            long length = new FileInfo(filename).Length;

            // attach filestream to binary reader
            BinaryReader binReader = new BinaryReader(fs);

            // read entire file into buffer
            fdata = new FileDataHandler(binReader, length);

            // close file reader
            fs.Close();
            fs.Dispose();
            binReader.Close();
        }
    }
}
