﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections;
using ElfViewerConsole.ELFModel.ELFFormat.Enums;

namespace ElfViewerConsole.ELFModel.ELFFormat
{
    class ELFSectionHeaderTables : IEnumerable
    {
        private ELFFile ElfFile;
        private ELFSectionHeaderTable[] SectionHeaderTable;
        public bool Exists { private set; get; }
        public int NumOfSectionHeaderTables { private set; get; }

        public ELFSectionHeaderTables(ELFFile elfFile)
        {
            this.ElfFile = elfFile;
        }

        internal void Read()
        {
            if ((ElfFile.ElfFileHeader.E_Shoff.Value != 0) &&
                (ElfFile.ElfFileHeader.E_Shoff.Value + (ulong)(ElfFile.ElfFileHeader.E_ShNum.Value * ElfFile.ElfFileHeader.E_ShEntSize.Value) <= (ulong)ElfFile.FileSize))
            {
                SectionHeaderTable = new ELFSectionHeaderTable[ElfFile.ElfFileHeader.E_ShNum.Value];
                for (int headerIndex = 0; headerIndex < ElfFile.ElfFileHeader.E_ShNum.Value; headerIndex++)
                {
                    SectionHeaderTable[headerIndex] = new ELFSectionHeaderTable(headerIndex, ElfFile);
                    SectionHeaderTable[headerIndex].Read();
                }
                Exists = true;
                NumOfSectionHeaderTables = ElfFile.ElfFileHeader.E_ShNum.Value;
            }
            else
            {
                Exists = false;
                NumOfSectionHeaderTables = 0;
            }
        }

        public ELFSectionHeaderTable this[int key]
        {
            get { return SectionHeaderTable[key]; }
        }

        public IEnumerator GetEnumerator()
        {
            foreach (ELFSectionHeaderTable sec in SectionHeaderTable)
            {
                yield return sec;
            }
        }
    }
}
