﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;

namespace ElfViewerConsole.ELFModel.ELFFormat.Enums
{
    [Flags]
    public enum EELFShFlags : uint
    {
        [Description("Contains writable data")]
        SHF_WRITE = 0x1,

        [Description("Allocated in memory image")]
        SHF_ALLOC = 0x2,

        [Description("Contains executable instructions")]
        SHF_EXECINSTR = 0x4,

        [Description("Can be merge to eliminate duplicate")]
        SHF_MERGE = 0x10,

        [Description("Contains null-terminated character strings")]
        SHF_STRINGS = 0x20,

        [Description("Sh_info field has section header table index")]
        SHF_INFO_LINK = 0x40,

        [Description("Contains special ordering requirements")]
        SHF_LINK_ORDER = 0x80,

        [Description("Requires requires special OS-specific processing")]
        SHF_OS_NONCONFORMING = 0x100,

        [Description("Member of a section group")]
        SHF_GROUP = 0x200,

        [Description("Contains thread-local storage")]
        SHF_TLS = 0x400,

        [Description("Reserved OS bit mask")]
        SHF_MASKOS = 0x0ff00000,

        [Description("Has more than 2 Gbyte")]
        SHF_AMD64_LARGE = 0x10000000,

        [Description("-")]
        SHF_ORDERED = 0x40000000,

        [Description("Excluded")]
        SHF_EXCLUDE = (uint)0x80000000,

        [Description("Reserved processor-specific bit mask")]
        SHF_MASKPROC = (uint)0xf0000000,
    }

    public static class EELFShFlagsExtension
    {
        public static string ToFriendlyString(this EELFShFlags val)
        {
            StringBuilder sb = new StringBuilder();

            if ((val & EELFShFlags.SHF_WRITE) != 0)
            {
                sb.Append("W");
            }
            if ((val & EELFShFlags.SHF_ALLOC) != 0)
            {
                sb.Append("A");
            }
            if ((val & EELFShFlags.SHF_EXECINSTR) != 0)
            {
                sb.Append("X");
            }
            if ((val & EELFShFlags.SHF_MERGE) != 0)
            {
                sb.Append("M");
            }
            if ((val & EELFShFlags.SHF_STRINGS) != 0)
            {
                sb.Append("S");
            }
            if ((val & EELFShFlags.SHF_INFO_LINK) != 0)
            {
                sb.Append("I");
            }
            if ((val & EELFShFlags.SHF_LINK_ORDER) != 0)
            {
                sb.Append("L");
            }
            if ((val & EELFShFlags.SHF_OS_NONCONFORMING) != 0)
            {
                sb.Append("O");
            }
            if ((val & EELFShFlags.SHF_GROUP) != 0)
            {
                sb.Append("G");
            }
            if ((val & EELFShFlags.SHF_TLS) != 0)
            {
                sb.Append("T");
            }
            if ((val & EELFShFlags.SHF_MASKOS) != 0)
            {
                sb.Append("M");
            }
            if ((val & EELFShFlags.SHF_AMD64_LARGE) != 0)
            {
                sb.Append("L");
            }
            if ((val & EELFShFlags.SHF_ORDERED) != 0)
            {
                sb.Append("o");
            }
            if ((val & EELFShFlags.SHF_EXCLUDE) != 0)
            {
                sb.Append("E");
            }
            if ((val & EELFShFlags.SHF_MASKPROC) != 0)
            {
                sb.Append("P");
            }

            if (sb.Length == 0)
            {
                sb.Append("-");
            }

            return sb.ToString();
        }
    }
}
