﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using ElfViewerConsole.ELFModel.ELFFormat;
using ElfViewerConsole.ELFModel.ELFFormat.Enums;

namespace ElfViewerConsole.ELFModel.ELFFormat
{
    interface IELFElem
    {
        /// <summary>
        /// thw actual value as general object
        /// </summary>
        Object ObjectValue { get; }

        /// <summary>
        /// absolute file offset of this member
        /// </summary>
        long Offset { get; }

        /// <summary>
        /// name of the ELF element
        /// </summary>
        string Name { get; }

        /// <summary>
        /// the actual value as formatted string
        /// </summary>
        /// <returns></returns>
        string ValueString();

        /// <summary>
        /// convert the numerical value to its ASCII representation, 
        /// e.g for value 0x54AD, "MZ" would be returned 
        /// </summary>
        /// <returns></returns>
        string NumValAsAscii();

        /// <summary>
        /// absolute file offset of this member as string
        /// </summary>
        string OffsetString { get; }

        /// <summary>
        /// the number of bytes of which the elements consists of
        /// </summary>
        /// <returns></returns>
        int SizeInBytes();

        /// <summary>
        /// returns if the ELFElem represents an ELF enum value
        /// </summary>
        /// <returns></returns>
        bool IsEnumValue();

        /// <summary>
        /// returns if the enum value is a flags enumeration.
        /// </summary>
        /// <returns></returns>
        bool HasEnumFlagsAttibrute();

        /// <summary>
        /// Get the enum value if elem is an enum type <seealso cref="IsEnumValue"/>
        /// </summary>
        /// <returns></returns>
        Enum GetEnum();

        /// <summary>
        /// Gets the detailed description string of the enum value from attributes.
        /// </summary>
        /// <returns></returns>
        string GetEnumDescription();
    }
}
